﻿using NetinaShop.Domain.Entities.Products;

namespace NetinaShop.Api.Controller;

public class ProductController : ICarterModule
{

    public virtual void AddRoutes(IEndpointRouteBuilder app)
    {
        var group = app.NewVersionedApi("Product")
            .MapGroup($"api/product")
            .RequireAuthorization(builder => builder.AddAuthenticationSchemes("Bearer").RequireAuthenticatedUser());

        group.MapGet("", GetAllAsync)
            .WithDisplayName("GetAllProducts")
            .HasApiVersion(1.0);

        group.MapGet("{id}", GetAsync)
            .WithDisplayName("GetProducts")
            .HasApiVersion(1.0);

        group.MapPost("", Post)
            .HasApiVersion(1.0);

        group.MapPut("", Put)
            .HasApiVersion(1.0);

        group.MapDelete("{id}", Delete)
            .HasApiVersion(1.0);
    }

    // GET:Get All Entity
    public async Task<IResult> GetAllAsync([FromQuery] int page, IMediator mediator, CancellationToken cancellationToken)
        => TypedResults.Ok(await mediator.Send(new GetProductsQuery(page),cancellationToken));

    // GET:Get An Entity By Id
    public async Task<IResult> GetAsync(Guid id, IMediator mediator,CancellationToken cancellationToken)
        => TypedResults.Ok(await mediator.Send(new GetProductQuery(id),cancellationToken));

    // POST:Create Entity
    public async Task<IResult> Post([FromBody] CreateProductCommand request, IMediator mediator,
        CancellationToken cancellationToken)
        => TypedResults.Ok(await mediator.Send(request, cancellationToken));

    // PUT:Update Entity
    public async Task<IResult> Put([FromBody] UpdateProductCommand request, IMediator mediator,
        CancellationToken cancellationToken)
        => TypedResults.Ok(await mediator.Send(request, cancellationToken));

    // DELETE:Delete Entity
    public async Task<IResult> Delete(Guid id, IMediator mediator, CancellationToken cancellationToken)
        => TypedResults.Ok(await mediator.Send(new DeleteProductCommand(id), cancellationToken));
}