﻿using NetinaShop.Domain.Entities.Categories;
using NetinaShop.Repository.Repositories.Base;
using NetinaShop.Repository.Repositories.Base.Contracts;

namespace NetinaShop.Repository.Handlers.Categories;

public class CreateCategoryCommandHandler : IRequestHandler<CreateCategoryCommand,CategoryLDto>
{
    private readonly IRepositoryWrapper _repositoryWrapper;

    public CreateCategoryCommandHandler(IRepositoryWrapper repositoryWrapper)
    {
        _repositoryWrapper = repositoryWrapper;
    }
    public async Task<CategoryLDto> Handle(CreateCategoryCommand request, CancellationToken cancellationToken)
    {
        var ent = Category.Create(request.Name, request.Description);
        if (request.ParentId != default)
            ent.SetParent(request.ParentId);
        foreach (var file in request.Files)
        {
            ent.AddFile(file.Name, file.FileLocation, file.FileName, file.IsHeader, file.IsPrimary, file.FileType);
        }
        _repositoryWrapper.SetRepository<Category>().Add(ent);
        await _repositoryWrapper.SaveChangesAsync(cancellationToken);
        return ent.AdaptToLDto();
    }
}