﻿using NetinaShop.Domain.Entities.Categories;

namespace NetinaShop.Repository.Handlers.Categories;

public class UpdateCategoryCommandHandler : IRequestHandler<UpdateCategoryCommand, bool>
{
    private readonly IRepositoryWrapper _repositoryWrapper;

    public UpdateCategoryCommandHandler(IRepositoryWrapper repositoryWrapper)
    {
        _repositoryWrapper = repositoryWrapper;
    }
    public async Task<bool> Handle(UpdateCategoryCommand request, CancellationToken cancellationToken)
    {
        var ent = await _repositoryWrapper.SetRepository<Category>().TableNoTracking
            .FirstOrDefaultAsync(c => c.Id == request.Id, cancellationToken);
        if (ent == null)
            throw new AppException("Category not found", ApiResultStatusCode.NotFound);

        var newEnt = Category.Create(request.Name, request.Description);
        newEnt.Id = ent.Id;
        if (request.ParentId != default)
            newEnt.SetParent(request.ParentId);
        

        var dbFiles = await _repositoryWrapper.SetRepository<CategoryStorageFile>().TableNoTracking
            .Where(s => s.CategoryId == newEnt.Id).ToListAsync(cancellationToken);
        foreach (var dbFile in dbFiles)
        {
            if (request.Files.Any(s => s.Id == dbFile.Id))
            {
                _repositoryWrapper.SetRepository<CategoryStorageFile>().Delete(dbFile);
                await _repositoryWrapper.SaveChangesAsync(cancellationToken);
            }
        }
        foreach (var file in request.Files.Where(f => f.Id == default))
        {
            newEnt.AddFile(file.Name, file.FileLocation, file.FileName, file.IsHeader, file.IsPrimary, file.FileType);
        }
        _repositoryWrapper.SetRepository<Category>().Add(newEnt);
        await _repositoryWrapper.SaveChangesAsync(cancellationToken);
        return true;
    }
}